package com.rjconsultores.ventaboletos.dao.hibernate;

import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Repository;

import com.rjconsultores.ventaboletos.dao.CasetaPeajeDAO;
import com.rjconsultores.ventaboletos.entidad.CasetaPeaje;
import com.rjconsultores.ventaboletos.entidad.CasetaPeajeExcepcion;

@Repository("casetaPeajeDAO")
public class CasetaPeajeHibernateDAO extends GenericHibernateDAO<CasetaPeaje, Integer> implements CasetaPeajeDAO {
	
	@Autowired
	public CasetaPeajeHibernateDAO(@Qualifier("sessionFactory") SessionFactory factory) {
		setSessionFactory(factory);
	}

	@Override
	public List<CasetaPeaje> obtenerTodos() {
		Criteria c = getSession().createCriteria(getPersistentClass());
		c.add(Restrictions.eq(ACTIVO, Boolean.TRUE));
		return (List<CasetaPeaje>) c.list();
	}
	
	public List<CasetaPeaje> buscarTodosOrdenadoPorDescricao() {
		Criteria c = getSession().createCriteria(getPersistentClass());
		c.add(Restrictions.eq(ACTIVO, Boolean.TRUE));
		c.addOrder(Order.asc("desccaseta"));
		c.setResultTransformer(Criteria.DISTINCT_ROOT_ENTITY);
		return (List<CasetaPeaje>) c.list();
	}
	
	@Override
	public List<CasetaPeaje> buscar(String casetaPeajeDesc) {
		Criteria c = getSession().createCriteria(getPersistentClass());
		c.add(Restrictions.eq(ACTIVO, Boolean.TRUE));
		c.add(Restrictions.eq("desccaseta", casetaPeajeDesc));
		return c.list();
	}

	@Override
	public void deletarVigencias(List<CasetaPeajeExcepcion> cpes) {
		for (CasetaPeajeExcepcion cpe : cpes) {
			String hql = " update CasetaPeajeExcepcion set activo = false, fecModif = :fecModif, usuarioId = :usuarioId where casetaPeajeExcepcionId = " + cpe.getCasetaPeajeExcepcionId();
			Query sq = getSession().createQuery(hql);
			sq.setTimestamp("fecModif", cpe.getFecmodif());
			sq.setInteger("usuarioId", cpe.getUsuarioId());
			sq.executeUpdate();
		}
	}
}