package com.rjconsultores.ventaboletos.service.impl;

import java.util.Calendar;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.rjconsultores.ventaboletos.dao.AutobusDocDAO;
import com.rjconsultores.ventaboletos.entidad.Autobus;
import com.rjconsultores.ventaboletos.entidad.AutobusDoc;
import com.rjconsultores.ventaboletos.service.AutobusDocService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;

@Service("autobusDocService")
public class AutobusDocServiceImpl implements AutobusDocService {

    @Autowired
    private AutobusDocDAO autobusDocDAO;

    public List<AutobusDoc> obtenerTodos() {
        return autobusDocDAO.obtenerTodos();
    }

    public AutobusDoc obtenerID(Integer id) {
        return autobusDocDAO.obtenerID(id);
    }
    
    public List<AutobusDoc> obtenerPorAutobus(Autobus autobus){
    	return autobusDocDAO.obtenerPorAutobus(autobus);
    }

    @Transactional
    public AutobusDoc suscribir(AutobusDoc entidad) {
        entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
        entidad.setFecmodif(Calendar.getInstance().getTime());
        entidad.setActivo(Boolean.TRUE);

        return autobusDocDAO.suscribir(entidad);
    }

    @Transactional
    public AutobusDoc actualizacion(AutobusDoc entidad) {
        entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
        entidad.setFecmodif(Calendar.getInstance().getTime());
        entidad.setActivo(Boolean.TRUE);

        return autobusDocDAO.actualizacion(entidad);
    }

    @Transactional
    public void borrar(AutobusDoc entidad) {
        
        entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
        entidad.setFecmodif(Calendar.getInstance().getTime());
        entidad.setActivo(Boolean.FALSE);

        autobusDocDAO.actualizacion(entidad);
    }

}
