package com.rjconsultores.ventaboletos.service.impl;

import java.util.Calendar;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import com.rjconsultores.ventaboletos.dao.ConfTotemDAO;
import com.rjconsultores.ventaboletos.entidad.ConfTotem;
import com.rjconsultores.ventaboletos.service.ConfTotemService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;

@Service("confTotemService")
public class ConfTotemServiceImpl implements ConfTotemService {

	@Autowired
	private ConfTotemDAO confTotemDAO;

	public List<ConfTotem> obtenerTodos() {
		return confTotemDAO.obtenerTodos();
	}

	public ConfTotem obtenerID(Integer id) {
		return confTotemDAO.obtenerID(id);
	}

	@Transactional(propagation = Propagation.SUPPORTS)
	public ConfTotem suscribir(ConfTotem entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado() != null ? UsuarioLogado.getUsuarioLogado().getUsuarioId() : null);
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);

		return confTotemDAO.suscribir(entidad);
	}

	@Transactional
	public ConfTotem actualizacion(ConfTotem entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);

		return confTotemDAO.actualizacion(entidad);
	}

	@Transactional
	public void borrar(ConfTotem entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.FALSE);

		confTotemDAO.actualizacion(entidad);
	}
	
	@Transactional(readOnly = true, propagation = Propagation.SUPPORTS)
	public ConfTotem buscar(String chave) {
		return confTotemDAO.buscar(chave);
	}
}
