
package com.rjconsultores.ventaboletos.service.impl;

import java.util.Calendar;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.rjconsultores.ventaboletos.dao.EmpresaCrediBancoConfigDAO;
import com.rjconsultores.ventaboletos.entidad.EmpresaCrediBancoConfig;
import com.rjconsultores.ventaboletos.service.EmpresaCrediBancoConfigService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;

@Service("empresaCrediBancoConfigService")
public class EmpresaCrediBancoConfigServiceImpl implements EmpresaCrediBancoConfigService {

	@Autowired
	private EmpresaCrediBancoConfigDAO empresaCrediBancoConfigDAO;

	public List<EmpresaCrediBancoConfig> obtenerTodos() {
		return empresaCrediBancoConfigDAO.obtenerTodos();
	}

	public EmpresaCrediBancoConfig obtenerID(Integer id) {
		return empresaCrediBancoConfigDAO.obtenerID(id);
	}

	@Transactional
	public EmpresaCrediBancoConfig suscribir(EmpresaCrediBancoConfig entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);

		return empresaCrediBancoConfigDAO.suscribir(entidad);
	}

	@Transactional
	public EmpresaCrediBancoConfig actualizacion(EmpresaCrediBancoConfig entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);

		return empresaCrediBancoConfigDAO.actualizacion(entidad);
	}

	@Transactional
	public void borrar(EmpresaCrediBancoConfig entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.FALSE);

		empresaCrediBancoConfigDAO.actualizacion(entidad);
	}

	public EmpresaCrediBancoConfig buscarChave(String chave, Integer empresaId) {
		return empresaCrediBancoConfigDAO.buscarChave(chave, empresaId);
	}

	public List<EmpresaCrediBancoConfig> buscarByEmpresa(Integer empresaId) {
		return empresaCrediBancoConfigDAO.buscarByEmpresa(empresaId);
	}

}
