package com.rjconsultores.ventaboletos.service.impl;

import java.util.Calendar;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.rjconsultores.ventaboletos.dao.EmpresaEmailDAO;
import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.EmpresaEmail;
import com.rjconsultores.ventaboletos.service.EmpresaEmailService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;

@Service("empresaEmailService")
public class EmpresaEmailServiceImpl implements EmpresaEmailService {

	@Autowired
	private EmpresaEmailDAO empresaEmailDAO;

	@Override
	public List<EmpresaEmail> obtenerTodos() {
		return empresaEmailDAO.obtenerTodos();
	}

	@Override
	public EmpresaEmail obtenerID(Integer id) {
		return empresaEmailDAO.obtenerID(id);
	}

	@Override
	@Transactional
	public EmpresaEmail suscribir(EmpresaEmail entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);

		return empresaEmailDAO.suscribir(entidad);
	}

	@Override
	@Transactional
	public EmpresaEmail actualizacion(EmpresaEmail entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);

		return empresaEmailDAO.actualizacion(entidad);
	}

	@Override
	@Transactional
	public void borrar(EmpresaEmail entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.FALSE);

		empresaEmailDAO.actualizacion(entidad);

	}

	@Override
	public EmpresaEmail buscarPorEmpresa(Empresa empresa) {
		return empresaEmailDAO.buscarPorEmpresa(empresa);
	}

}
