package com.rjconsultores.ventaboletos.service.impl;

import java.util.Calendar;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.rjconsultores.ventaboletos.dao.EmpresaSaferConfigDAO;
import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.EmpresaSaferConfig;
import com.rjconsultores.ventaboletos.service.EmpresaSaferConfigService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;

@Service("empresaSaferConfigService")
public class EmpresaSaferConfigServiceImpl implements EmpresaSaferConfigService {

	@Autowired
	private EmpresaSaferConfigDAO empresaSaferConfigDAO;

	@Override
	public List<EmpresaSaferConfig> obtenerTodos() {
		return empresaSaferConfigDAO.obtenerTodos();
	}

	@Override
	public EmpresaSaferConfig obtenerID(Integer id) {
		return empresaSaferConfigDAO.obtenerID(id);
	}

	@Override
	@Transactional
	public EmpresaSaferConfig suscribir(EmpresaSaferConfig entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);

		return empresaSaferConfigDAO.suscribir(entidad);
	}

	@Override
	@Transactional
	public EmpresaSaferConfig actualizacion(EmpresaSaferConfig entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);

		return empresaSaferConfigDAO.actualizacion(entidad);
	}

	@Override
	@Transactional
	public void borrar(EmpresaSaferConfig entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.FALSE);

		empresaSaferConfigDAO.actualizacion(entidad);

	}

	@Override
	public EmpresaSaferConfig buscarPorEmpresa(Empresa empresa) {
		return empresaSaferConfigDAO.buscarPorEmpresa(empresa);
	}

}
