package com.rjconsultores.ventaboletos.service.impl;

import java.util.Calendar;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.rjconsultores.ventaboletos.dao.PricingFormapagoDAO;
import com.rjconsultores.ventaboletos.entidad.Pricing;
import com.rjconsultores.ventaboletos.entidad.PricingFormapago;
import com.rjconsultores.ventaboletos.service.PricingFormapagoService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;

@Service("pricingFormapagoService")
public class PricingFormapagoServiceImpl implements PricingFormapagoService {

	@Autowired
	private PricingFormapagoDAO pricingFormapagoDAO;

	@Override
	public List<PricingFormapago> obtenerTodos() {
		return pricingFormapagoDAO.obtenerTodos();
	}

	@Override
	public PricingFormapago obtenerID(Integer id) {
		return pricingFormapagoDAO.obtenerID(id);
	}

	@Transactional
	public PricingFormapago suscribir(PricingFormapago entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Pricing.ATIVO);

		return pricingFormapagoDAO.suscribir(entidad);

	}

	@Transactional
	public PricingFormapago actualizacion(PricingFormapago entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Pricing.ATIVO);

		return pricingFormapagoDAO.actualizacion(entidad);
	}

	@Transactional
	public void borrar(PricingFormapago entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Pricing.EXCLUIDO);

		pricingFormapagoDAO.actualizacion(entidad);
	}

}
